@ Blinken lassen von 4 LEDs mit verschiedenen Frequenzen
@.equ LINUX_VERSION, 1  @ auskommentieren fuer "bare-metal"

.ifdef LINUX_VERSION
        .text
        .global meinunterprogramm
meinunterprogramm:
        push {r1-r12,lr}
        mov r7, r0           @ GPIO-Adresse im r7 behalten
	ldr r8, adrTimeradr  @ Adresse der Adresse laden
	str r1, [r8,#0]      @ Timer-Basisadresse in Timeradr speichern
        bl meinstart
        mov r0, #0           @ Rueckgabewert
        pop {r1-r12,pc}
meinstart:
.else
	.section .init
	.text
	.global _start
_start:	
        mov sp, #0x8000          @ Stack Pointer setzen
        mov r7, #0x20000000
        add r7, #0x200000        @ GPIO-Adresse im r7 behalten
.endif
        bl gpio_initialisieren
        ldr r4, adrWartezeiten   @ Die Adresse von Wartezeiten ins r4 laden, nicht der Inhalt
        ldr r0, [r4],#4          @ 1. Wartezeit einlesen
        ldr r1, [r4],#4          @ 2. Wartezeit einlesen
        ldr r2, [r4],#4          @ 3. Wartezeit einlesen
        ldr r3, [r4],#4          @ 4. Wartezeit einlesen
        mov r5, #0xF         @ r5 setzen um beim 1. Mal in pause alle Vergleichsregister zu setzen
        mov r6, #0xF         @ beim Start alle 4 LEDs ein
L1:
        bl leds_gemaess_r6_schalten
        bl pause
        b L1

Wartezeiten:
        .word 500000       @ 0.5 Sekunden, also 1 Hz Blinkfrequenz
        .word 600000       @ 0.6 Sec, also etwas langsamer blinken, etwa 0.833 Hz
        .word 700000       @ 0.7 Sec, etwa 0.714 Hz
        .word 800000       @ 0.8 Sec, genau 0.625 Hz
adrWartezeiten:
        .word Wartezeiten  @ die Adresse von Wartezeiten

gpio_initialisieren:    @ alle am GPIO angeschlossenen LEDs auf Ausgang setzen
                        @ r7=GPIO-Basisadresse schon gesetzt
        push {r0,lr}         @ Register retten
        mov r0, #(1<<(9*3))  @ GPIO09 blaue LED
        str r0, [r7,#0]      @ benutzte GPIO im Bereich 00-09 setzen
        mov r0, #(1<<(6*3))  @ GPIO16 gruen OK-LED
        add r0, #(1<<(7*3))  @ GPIO17 rote LED
        add r0, #1           @ GPIO10 gruene LED
        str r0, [r7,#4]      @ benutzte GPIO im Bereich 10-19 setzen
        mov r0, #(1<<(7*3))  @ GPIO27 gelbe LED
        add r0, #(1<<(2*3))  @ GPIO22 orange LED
        str r0, [r7,#8]      @ benutzte GPIO im Bereich 20-29 setzen
        pop {r0,pc}          @ gerettete Register zurueckholen, und Ruecksprung

	.data
Timeradr:
        .word 0x20003000     @ Timer-Basis-Adresse als konstanter Zahlenwert
	.text
adrTimeradr:
	.word Timeradr	     @ die Adresse der Adresse des Timers

pause:  @ Eingabeparameter: r0-r3 = Wartezeiten, r5=0 oder 0xF, r6=LED-Zustaende
        @ Rueckgabeparameter: r5=0, r6=neue LED-Zustaende
        push {r4,r7,lr}      @ Register retten
        ldr r8, adrTimeradr  @ Adresse der Adresse laden
	ldr r8, [r8,#0]      @ Timer-Basisadresse laden
        tst r5, #0xF         @ r5 beim ersten Aufruf gesetzt?
        bne L3               @ ja-> Warteschlaufe ueberspringen
L2:	ldr r5, [r8,#0]      @ neuer Status abfragen
        ands r5, #0xF        @ ist eins von M0-M3 gesetzt?
        beq L2               @ nein-> warten bis mindestens eins gesetzt
L3:	ldr r4, [r8,#4]      @ Zaehler des Timers einlesen
        tst r5, #1           @ ist M0 gesetzt?
        addne r7, r4, r0     @ ja: neuer Endzeitpunkt fuer 1. Wartezeit (r7=r4+r0)
        strne r7, [r8,#0xC]  @ ja: speichern im Vergleichsregister "Compare 0"
        tst r5, #2           @ ist M1 gesetzt?
        addne r7, r4, r1     @ neuer Endzeitpunkt fuer 2. Wartezeit
        strne r7, [r8,#0x10] @ speichern im Vergleichsregister "Compare 1"
        tst r5, #4           @ ist M2 gesetzt?
        addne r7, r4, r2     @ neuer Endzeitpunkt fuer 3. Wartezeit
        strne r7, [r8,#0x14] @ speichern im Vergleichsregister "Compare 2"
        tst r5, #8           @ ist M3 gesetzt?
        addne r7, r4, r3     @ neuer Endzeitpunkt fuer 4. Wartezeit
        strne r7, [r8,#0x18] @ speichern im Vergleichsregister "Compare 3"
        str r5, [r8,#0]      @ in r5 gesetzte Bits im CS ruecksetzen
        eor r6, r5           @ r6 aktualisieren: die Bits, die im r5 gesetzt sind im r6 umkehren
        mov r5, #0           @ r5 loeschen fuer naechsten Durchlauf
        pop {r4,r7,pc}       @ gerettete Register zurueckholen, und Ruecksprung

leds_gemaess_r6_schalten:
        push {r0,r1,lr}     @ Register retten
        mvn r1, r6          @ alle Bits umkehren, da LEDs mit 0 eingeschaltet werden
        mov r0, #17         @ GPIO17 1. LED, rot,  1.Bit im r1 schon richtig
        bl gpio_ein_aus
        mov r0, #27         @ GPIO27 2. LED, gelb
        lsr r1, #1          @ naechstes Bit im r1 zurechtschieben
        bl gpio_ein_aus
        mov r0, #22         @ GPIO22 3. LED, gruen
        lsr r1, #1          @ naechstes Bit im r1 zurechtschieben
        bl gpio_ein_aus
        mov r0, #10         @ GPIO10 4. LED, orange
        lsr r1, #1          @ naechstes Bit im r1 zurechtschieben
        bl gpio_ein_aus
        mov r0, #9          @ GPIO09 5. LED, blau
        lsr r1, #1          @ naechstes Bit im r1 zurechtschieben
        bl gpio_ein_aus
        mov r0, #16         @ GPIO16 6. LED, gruene ok-LED, hier aendern fuer andere
        lsr r1, #1          @ naechstes Bit im r1 zurechtschieben
        bl gpio_ein_aus
        pop {r0,r1,pc}      @ gerettete Register zurueckholen, und Ruecksprung

gpio_ein_aus:  @ Parameter: r0=Nummer, r1.bit0=Spannung ein/aus (0=aus, 1=ein), r7=GPIO-Basisadresse
        push {r2,lr}        @ Register retten
        mov r2, #1
        lsl r2, r0          @ r2 = (1<<r0)
        tst r1, #1          @ ist erstes Bit in r1 gleich Null?
        streq r2, [r7,#40]  @ ja: Spannung aus
        strne r2, [r7,#28]  @ nein: Spannung ein
        pop {r2,pc}         @ gerettete Register zurueckholen, und Ruecksprung
